"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import sys
import unittest

import apideck
from apideck.model.allocation import Allocation
from apideck.model.currency import Currency
from apideck.model.custom_field import CustomField
from apideck.model.deprecated_linked_supplier import DeprecatedLinkedSupplier
from apideck.model.linked_customer import LinkedCustomer
from apideck.model.linked_ledger_account import LinkedLedgerAccount
from apideck.model.linked_tracking_categories import LinkedTrackingCategories
from apideck.model.pass_through_body import PassThroughBody
from apideck.model.payment_status import PaymentStatus
from apideck.model.payment_type import PaymentType
globals()['Allocation'] = Allocation
globals()['Currency'] = Currency
globals()['CustomField'] = CustomField
globals()['DeprecatedLinkedSupplier'] = DeprecatedLinkedSupplier
globals()['LinkedCustomer'] = LinkedCustomer
globals()['LinkedLedgerAccount'] = LinkedLedgerAccount
globals()['LinkedTrackingCategories'] = LinkedTrackingCategories
globals()['PassThroughBody'] = PassThroughBody
globals()['PaymentStatus'] = PaymentStatus
globals()['PaymentType'] = PaymentType
from apideck.model.payment import Payment


class TestPayment(unittest.TestCase):
    """Payment unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testPayment(self):
        """Test Payment"""
        # FIXME: construct object with mandatory attributes with example values
        # model = Payment()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()



"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from apideck.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from apideck.exceptions import ApiAttributeError


def lazy_import():
    from apideck.model.currency import Currency
    from apideck.model.idempotency_key import IdempotencyKey
    from apideck.model.order_customers import OrderCustomers
    from apideck.model.order_discounts import OrderDiscounts
    from apideck.model.order_fulfillments import OrderFulfillments
    from apideck.model.order_line_items import OrderLineItems
    from apideck.model.order_payments import OrderPayments
    from apideck.model.order_refunds import OrderRefunds
    from apideck.model.order_tenders import OrderTenders
    from apideck.model.pass_through_body import PassThroughBody
    from apideck.model.service_charges import ServiceCharges
    globals()['Currency'] = Currency
    globals()['IdempotencyKey'] = IdempotencyKey
    globals()['OrderCustomers'] = OrderCustomers
    globals()['OrderDiscounts'] = OrderDiscounts
    globals()['OrderFulfillments'] = OrderFulfillments
    globals()['OrderLineItems'] = OrderLineItems
    globals()['OrderPayments'] = OrderPayments
    globals()['OrderRefunds'] = OrderRefunds
    globals()['OrderTenders'] = OrderTenders
    globals()['PassThroughBody'] = PassThroughBody
    globals()['ServiceCharges'] = ServiceCharges


class Order(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('status',): {
            'OPEN': "open",
            'DRAFT': "draft",
            'DELIVERED': "delivered",
            'DELAYED': "delayed",
            'VOIDED': "voided",
            'COMPLETED': "completed",
            'HIDDEN': "hidden",
        },
        ('payment_status',): {
            'OPEN': "open",
            'PAID': "paid",
            'REFUNDED': "refunded",
            'CREDITED': "credited",
            'PARTIALLY_PAID': "partially_paid",
            'PARTIALLY_REFUNDED': "partially_refunded",
            'UNKNOWN': "unknown",
        },
        ('source',): {
            'None': None,
            'IN-STORE': "in-store",
            'ONLINE': "online",
            'OPT': "opt",
            'API': "api",
            'KIOSK': "kiosk",
            'CALLER-ID': "caller-id",
            'GOOGLE': "google",
            'INVOICE': "invoice",
        },
    }

    validations = {
    }

    additional_properties_type = None

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'merchant_id': (str,),  # noqa: E501
            'location_id': (str,),  # noqa: E501
            'id': (str,),  # noqa: E501
            'idempotency_key': (IdempotencyKey,),  # noqa: E501
            'order_number': (str,),  # noqa: E501
            'order_date': (date, none_type,),  # noqa: E501
            'closed_date': (date, none_type,),  # noqa: E501
            'reference_id': (str, none_type,),  # noqa: E501
            'status': (str,),  # noqa: E501
            'payment_status': (str,),  # noqa: E501
            'currency': (Currency,),  # noqa: E501
            'title': (str,),  # noqa: E501
            'note': (str,),  # noqa: E501
            'customer_id': (str,),  # noqa: E501
            'employee_id': (str,),  # noqa: E501
            'order_type_id': (str,),  # noqa: E501
            'table': (str,),  # noqa: E501
            'seat': (str,),  # noqa: E501
            'total_amount': (int, none_type,),  # noqa: E501
            'total_tip': (int, none_type,),  # noqa: E501
            'total_tax': (int, none_type,),  # noqa: E501
            'total_discount': (int, none_type,),  # noqa: E501
            'total_refund': (int, none_type,),  # noqa: E501
            'total_service_charge': (int, none_type,),  # noqa: E501
            'refunded': (bool,),  # noqa: E501
            'customers': ([OrderCustomers],),  # noqa: E501
            'fulfillments': ([OrderFulfillments],),  # noqa: E501
            'line_items': ([OrderLineItems],),  # noqa: E501
            'payments': ([OrderPayments],),  # noqa: E501
            'service_charges': (ServiceCharges,),  # noqa: E501
            'refunds': ([OrderRefunds],),  # noqa: E501
            'taxes': ([bool, date, datetime, dict, float, int, list, str, none_type],),  # noqa: E501
            'discounts': ([OrderDiscounts],),  # noqa: E501
            'tenders': ([OrderTenders],),  # noqa: E501
            'source': (str, none_type,),  # noqa: E501
            'voided': (bool,),  # noqa: E501
            'voided_at': (datetime,),  # noqa: E501
            'custom_mappings': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type,),  # noqa: E501
            'version': (str, none_type,),  # noqa: E501
            'updated_by': (str, none_type,),  # noqa: E501
            'created_by': (str, none_type,),  # noqa: E501
            'updated_at': (datetime, none_type,),  # noqa: E501
            'created_at': (datetime, none_type,),  # noqa: E501
            'pass_through': (PassThroughBody,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'merchant_id': 'merchant_id',  # noqa: E501
        'location_id': 'location_id',  # noqa: E501
        'id': 'id',  # noqa: E501
        'idempotency_key': 'idempotency_key',  # noqa: E501
        'order_number': 'order_number',  # noqa: E501
        'order_date': 'order_date',  # noqa: E501
        'closed_date': 'closed_date',  # noqa: E501
        'reference_id': 'reference_id',  # noqa: E501
        'status': 'status',  # noqa: E501
        'payment_status': 'payment_status',  # noqa: E501
        'currency': 'currency',  # noqa: E501
        'title': 'title',  # noqa: E501
        'note': 'note',  # noqa: E501
        'customer_id': 'customer_id',  # noqa: E501
        'employee_id': 'employee_id',  # noqa: E501
        'order_type_id': 'order_type_id',  # noqa: E501
        'table': 'table',  # noqa: E501
        'seat': 'seat',  # noqa: E501
        'total_amount': 'total_amount',  # noqa: E501
        'total_tip': 'total_tip',  # noqa: E501
        'total_tax': 'total_tax',  # noqa: E501
        'total_discount': 'total_discount',  # noqa: E501
        'total_refund': 'total_refund',  # noqa: E501
        'total_service_charge': 'total_service_charge',  # noqa: E501
        'refunded': 'refunded',  # noqa: E501
        'customers': 'customers',  # noqa: E501
        'fulfillments': 'fulfillments',  # noqa: E501
        'line_items': 'line_items',  # noqa: E501
        'payments': 'payments',  # noqa: E501
        'service_charges': 'service_charges',  # noqa: E501
        'refunds': 'refunds',  # noqa: E501
        'taxes': 'taxes',  # noqa: E501
        'discounts': 'discounts',  # noqa: E501
        'tenders': 'tenders',  # noqa: E501
        'source': 'source',  # noqa: E501
        'voided': 'voided',  # noqa: E501
        'voided_at': 'voided_at',  # noqa: E501
        'custom_mappings': 'custom_mappings',  # noqa: E501
        'version': 'version',  # noqa: E501
        'updated_by': 'updated_by',  # noqa: E501
        'created_by': 'created_by',  # noqa: E501
        'updated_at': 'updated_at',  # noqa: E501
        'created_at': 'created_at',  # noqa: E501
        'pass_through': 'pass_through',  # noqa: E501
    }

    read_only_vars = {
        'id',  # noqa: E501
        'source',  # noqa: E501
        'voided_at',  # noqa: E501
        'custom_mappings',  # noqa: E501
        'updated_by',  # noqa: E501
        'created_by',  # noqa: E501
        'updated_at',  # noqa: E501
        'created_at',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, merchant_id, location_id, *args, **kwargs):  # noqa: E501
        """Order - a model defined in OpenAPI

        Args:
            merchant_id (str):
            location_id (str):

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): A unique identifier for an object.. [optional]  # noqa: E501
            idempotency_key (IdempotencyKey): [optional]  # noqa: E501
            order_number (str): [optional]  # noqa: E501
            order_date (date, none_type): [optional]  # noqa: E501
            closed_date (date, none_type): [optional]  # noqa: E501
            reference_id (str, none_type): An optional user-defined reference ID that associates this record with another entity in an external system. For example, a customer ID from an external customer management system.. [optional]  # noqa: E501
            status (str): Order status. Clover specific: If no value is set, the status defaults to hidden, which indicates a hidden order. A hidden order is not displayed in user interfaces and can only be retrieved by its id. When creating an order via the REST API the value must be manually set to 'open'. More info [https://docs.clover.com/reference/orderupdateorder](). [optional]  # noqa: E501
            payment_status (str): Is this order paid or not?. [optional]  # noqa: E501
            currency (Currency): [optional]  # noqa: E501
            title (str): [optional]  # noqa: E501
            note (str): A note with information about this order, may be printed on the order receipt and displayed in apps. [optional]  # noqa: E501
            customer_id (str): [optional]  # noqa: E501
            employee_id (str): [optional]  # noqa: E501
            order_type_id (str): [optional]  # noqa: E501
            table (str): [optional]  # noqa: E501
            seat (str): [optional]  # noqa: E501
            total_amount (int, none_type): [optional]  # noqa: E501
            total_tip (int, none_type): [optional]  # noqa: E501
            total_tax (int, none_type): [optional]  # noqa: E501
            total_discount (int, none_type): [optional]  # noqa: E501
            total_refund (int, none_type): [optional]  # noqa: E501
            total_service_charge (int, none_type): [optional]  # noqa: E501
            refunded (bool): [optional]  # noqa: E501
            customers ([OrderCustomers]): [optional]  # noqa: E501
            fulfillments ([OrderFulfillments]): [optional]  # noqa: E501
            line_items ([OrderLineItems]): [optional]  # noqa: E501
            payments ([OrderPayments]): [optional]  # noqa: E501
            service_charges (ServiceCharges): [optional]  # noqa: E501
            refunds ([OrderRefunds]): [optional]  # noqa: E501
            taxes ([bool, date, datetime, dict, float, int, list, str, none_type]): [optional]  # noqa: E501
            discounts ([OrderDiscounts]): [optional]  # noqa: E501
            tenders ([OrderTenders]): [optional]  # noqa: E501
            source (str, none_type): Source of order. Indicates the way that the order was placed.. [optional]  # noqa: E501
            voided (bool): [optional]  # noqa: E501
            voided_at (datetime): [optional]  # noqa: E501
            custom_mappings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): When custom mappings are configured on the resource, the result is included here.. [optional]  # noqa: E501
            version (str, none_type): [optional]  # noqa: E501
            updated_by (str, none_type): The user who last updated the object.. [optional]  # noqa: E501
            created_by (str, none_type): The user who created the object.. [optional]  # noqa: E501
            updated_at (datetime, none_type): The date and time when the object was last updated.. [optional]  # noqa: E501
            created_at (datetime, none_type): The date and time when the object was created.. [optional]  # noqa: E501
            pass_through (PassThroughBody): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.merchant_id = merchant_id
        self.location_id = location_id
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, merchant_id, location_id, *args, **kwargs):  # noqa: E501
        """Order - a model defined in OpenAPI

        Args:
            merchant_id (str):
            location_id (str):

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): A unique identifier for an object.. [optional]  # noqa: E501
            idempotency_key (IdempotencyKey): [optional]  # noqa: E501
            order_number (str): [optional]  # noqa: E501
            order_date (date, none_type): [optional]  # noqa: E501
            closed_date (date, none_type): [optional]  # noqa: E501
            reference_id (str, none_type): An optional user-defined reference ID that associates this record with another entity in an external system. For example, a customer ID from an external customer management system.. [optional]  # noqa: E501
            status (str): Order status. Clover specific: If no value is set, the status defaults to hidden, which indicates a hidden order. A hidden order is not displayed in user interfaces and can only be retrieved by its id. When creating an order via the REST API the value must be manually set to 'open'. More info [https://docs.clover.com/reference/orderupdateorder](). [optional]  # noqa: E501
            payment_status (str): Is this order paid or not?. [optional]  # noqa: E501
            currency (Currency): [optional]  # noqa: E501
            title (str): [optional]  # noqa: E501
            note (str): A note with information about this order, may be printed on the order receipt and displayed in apps. [optional]  # noqa: E501
            customer_id (str): [optional]  # noqa: E501
            employee_id (str): [optional]  # noqa: E501
            order_type_id (str): [optional]  # noqa: E501
            table (str): [optional]  # noqa: E501
            seat (str): [optional]  # noqa: E501
            total_amount (int, none_type): [optional]  # noqa: E501
            total_tip (int, none_type): [optional]  # noqa: E501
            total_tax (int, none_type): [optional]  # noqa: E501
            total_discount (int, none_type): [optional]  # noqa: E501
            total_refund (int, none_type): [optional]  # noqa: E501
            total_service_charge (int, none_type): [optional]  # noqa: E501
            refunded (bool): [optional]  # noqa: E501
            customers ([OrderCustomers]): [optional]  # noqa: E501
            fulfillments ([OrderFulfillments]): [optional]  # noqa: E501
            line_items ([OrderLineItems]): [optional]  # noqa: E501
            payments ([OrderPayments]): [optional]  # noqa: E501
            service_charges (ServiceCharges): [optional]  # noqa: E501
            refunds ([OrderRefunds]): [optional]  # noqa: E501
            taxes ([bool, date, datetime, dict, float, int, list, str, none_type]): [optional]  # noqa: E501
            discounts ([OrderDiscounts]): [optional]  # noqa: E501
            tenders ([OrderTenders]): [optional]  # noqa: E501
            source (str, none_type): Source of order. Indicates the way that the order was placed.. [optional]  # noqa: E501
            voided (bool): [optional]  # noqa: E501
            voided_at (datetime): [optional]  # noqa: E501
            custom_mappings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): When custom mappings are configured on the resource, the result is included here.. [optional]  # noqa: E501
            version (str, none_type): [optional]  # noqa: E501
            updated_by (str, none_type): The user who last updated the object.. [optional]  # noqa: E501
            created_by (str, none_type): The user who created the object.. [optional]  # noqa: E501
            updated_at (datetime, none_type): The date and time when the object was last updated.. [optional]  # noqa: E501
            created_at (datetime, none_type): The date and time when the object was created.. [optional]  # noqa: E501
            pass_through (PassThroughBody): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.merchant_id = merchant_id
        self.location_id = location_id
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import sys
import unittest

import apideck
from apideck.model.currency import Currency
from apideck.model.ecommerce_customer_addresses import EcommerceCustomerAddresses
from apideck.model.email import Email
from apideck.model.linked_ecommerce_order import LinkedEcommerceOrder
from apideck.model.phone_number import PhoneNumber
globals()['Currency'] = Currency
globals()['EcommerceCustomerAddresses'] = EcommerceCustomerAddresses
globals()['Email'] = Email
globals()['LinkedEcommerceOrder'] = LinkedEcommerceOrder
globals()['PhoneNumber'] = PhoneNumber
from apideck.model.ecommerce_customer import EcommerceCustomer


class TestEcommerceCustomer(unittest.TestCase):
    """EcommerceCustomer unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testEcommerceCustomer(self):
        """Test EcommerceCustomer"""
        # FIXME: construct object with mandatory attributes with example values
        # model = EcommerceCustomer()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()



"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from apideck.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from apideck.exceptions import ApiAttributeError


def lazy_import():
    from apideck.model.activity_attendee import ActivityAttendee
    from apideck.model.address import Address
    from apideck.model.custom_field import CustomField
    from apideck.model.pass_through_body import PassThroughBody
    globals()['ActivityAttendee'] = ActivityAttendee
    globals()['Address'] = Address
    globals()['CustomField'] = CustomField
    globals()['PassThroughBody'] = PassThroughBody


class Activity(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('type',): {
            'None': None,
            'CALL': "call",
            'MEETING': "meeting",
            'EMAIL': "email",
            'NOTE': "note",
            'TASK': "task",
            'DEADLINE': "deadline",
            'SEND-LETTER': "send-letter",
            'SEND-QUOTE': "send-quote",
            'OTHER': "other",
        },
        ('show_as',): {
            'None': None,
            'FREE': "free",
            'BUSY': "busy",
        },
    }

    validations = {
        ('duration_seconds',): {
            'inclusive_minimum': 0,
        },
    }

    additional_properties_type = None

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'type': (str, none_type,),  # noqa: E501
            'id': (str,),  # noqa: E501
            'downstream_id': (str, none_type,),  # noqa: E501
            'activity_datetime': (str, none_type,),  # noqa: E501
            'duration_seconds': (int, none_type,),  # noqa: E501
            'user_id': (str, none_type,),  # noqa: E501
            'account_id': (str, none_type,),  # noqa: E501
            'contact_id': (str, none_type,),  # noqa: E501
            'company_id': (str, none_type,),  # noqa: E501
            'opportunity_id': (str, none_type,),  # noqa: E501
            'lead_id': (str, none_type,),  # noqa: E501
            'owner_id': (str, none_type,),  # noqa: E501
            'campaign_id': (str, none_type,),  # noqa: E501
            'case_id': (str, none_type,),  # noqa: E501
            'asset_id': (str, none_type,),  # noqa: E501
            'contract_id': (str, none_type,),  # noqa: E501
            'product_id': (str, none_type,),  # noqa: E501
            'solution_id': (str, none_type,),  # noqa: E501
            'custom_object_id': (str, none_type,),  # noqa: E501
            'title': (str, none_type,),  # noqa: E501
            'description': (str, none_type,),  # noqa: E501
            'note': (str, none_type,),  # noqa: E501
            'location': (str, none_type,),  # noqa: E501
            'location_address': (Address,),  # noqa: E501
            'all_day_event': (bool, none_type,),  # noqa: E501
            'private': (bool, none_type,),  # noqa: E501
            'group_event': (bool, none_type,),  # noqa: E501
            'event_sub_type': (str, none_type,),  # noqa: E501
            'group_event_type': (str, none_type,),  # noqa: E501
            'child': (bool, none_type,),  # noqa: E501
            'archived': (bool, none_type,),  # noqa: E501
            'deleted': (bool, none_type,),  # noqa: E501
            'show_as': (str, none_type,),  # noqa: E501
            'done': (bool, none_type,),  # noqa: E501
            'start_datetime': (str, none_type,),  # noqa: E501
            'end_datetime': (str, none_type,),  # noqa: E501
            'duration_minutes': (int, none_type,),  # noqa: E501
            'activity_date': (str, none_type,),  # noqa: E501
            'end_date': (str, none_type,),  # noqa: E501
            'recurrent': (bool,),  # noqa: E501
            'reminder_datetime': (str, none_type,),  # noqa: E501
            'reminder_set': (bool, none_type,),  # noqa: E501
            'video_conference_url': (str, none_type,),  # noqa: E501
            'video_conference_id': (str, none_type,),  # noqa: E501
            'custom_fields': ([CustomField],),  # noqa: E501
            'attendees': ([ActivityAttendee],),  # noqa: E501
            'custom_mappings': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type,),  # noqa: E501
            'updated_by': (str, none_type,),  # noqa: E501
            'created_by': (str, none_type,),  # noqa: E501
            'updated_at': (str, none_type,),  # noqa: E501
            'created_at': (str, none_type,),  # noqa: E501
            'pass_through': (PassThroughBody,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'type': 'type',  # noqa: E501
        'id': 'id',  # noqa: E501
        'downstream_id': 'downstream_id',  # noqa: E501
        'activity_datetime': 'activity_datetime',  # noqa: E501
        'duration_seconds': 'duration_seconds',  # noqa: E501
        'user_id': 'user_id',  # noqa: E501
        'account_id': 'account_id',  # noqa: E501
        'contact_id': 'contact_id',  # noqa: E501
        'company_id': 'company_id',  # noqa: E501
        'opportunity_id': 'opportunity_id',  # noqa: E501
        'lead_id': 'lead_id',  # noqa: E501
        'owner_id': 'owner_id',  # noqa: E501
        'campaign_id': 'campaign_id',  # noqa: E501
        'case_id': 'case_id',  # noqa: E501
        'asset_id': 'asset_id',  # noqa: E501
        'contract_id': 'contract_id',  # noqa: E501
        'product_id': 'product_id',  # noqa: E501
        'solution_id': 'solution_id',  # noqa: E501
        'custom_object_id': 'custom_object_id',  # noqa: E501
        'title': 'title',  # noqa: E501
        'description': 'description',  # noqa: E501
        'note': 'note',  # noqa: E501
        'location': 'location',  # noqa: E501
        'location_address': 'location_address',  # noqa: E501
        'all_day_event': 'all_day_event',  # noqa: E501
        'private': 'private',  # noqa: E501
        'group_event': 'group_event',  # noqa: E501
        'event_sub_type': 'event_sub_type',  # noqa: E501
        'group_event_type': 'group_event_type',  # noqa: E501
        'child': 'child',  # noqa: E501
        'archived': 'archived',  # noqa: E501
        'deleted': 'deleted',  # noqa: E501
        'show_as': 'show_as',  # noqa: E501
        'done': 'done',  # noqa: E501
        'start_datetime': 'start_datetime',  # noqa: E501
        'end_datetime': 'end_datetime',  # noqa: E501
        'duration_minutes': 'duration_minutes',  # noqa: E501
        'activity_date': 'activity_date',  # noqa: E501
        'end_date': 'end_date',  # noqa: E501
        'recurrent': 'recurrent',  # noqa: E501
        'reminder_datetime': 'reminder_datetime',  # noqa: E501
        'reminder_set': 'reminder_set',  # noqa: E501
        'video_conference_url': 'video_conference_url',  # noqa: E501
        'video_conference_id': 'video_conference_id',  # noqa: E501
        'custom_fields': 'custom_fields',  # noqa: E501
        'attendees': 'attendees',  # noqa: E501
        'custom_mappings': 'custom_mappings',  # noqa: E501
        'updated_by': 'updated_by',  # noqa: E501
        'created_by': 'created_by',  # noqa: E501
        'updated_at': 'updated_at',  # noqa: E501
        'created_at': 'created_at',  # noqa: E501
        'pass_through': 'pass_through',  # noqa: E501
    }

    read_only_vars = {
        'id',  # noqa: E501
        'downstream_id',  # noqa: E501
        'duration_minutes',  # noqa: E501
        'custom_mappings',  # noqa: E501
        'updated_by',  # noqa: E501
        'created_by',  # noqa: E501
        'updated_at',  # noqa: E501
        'created_at',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, type, *args, **kwargs):  # noqa: E501
        """Activity - a model defined in OpenAPI

        Args:
            type (str, none_type): The type of the activity

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): The unique identifier of the activity. [optional]  # noqa: E501
            downstream_id (str, none_type): The third-party API ID of original entity. [optional]  # noqa: E501
            activity_datetime (str, none_type): The date and time of the activity. [optional]  # noqa: E501
            duration_seconds (int, none_type): The duration of the activity in seconds. [optional]  # noqa: E501
            user_id (str, none_type): The user related to the activity. [optional]  # noqa: E501
            account_id (str, none_type): The account related to the activity. [optional]  # noqa: E501
            contact_id (str, none_type): The contact related to the activity. [optional]  # noqa: E501
            company_id (str, none_type): The company related to the activity. [optional]  # noqa: E501
            opportunity_id (str, none_type): The opportunity related to the activity. [optional]  # noqa: E501
            lead_id (str, none_type): The lead related to the activity. [optional]  # noqa: E501
            owner_id (str, none_type): The owner of the activity. [optional]  # noqa: E501
            campaign_id (str, none_type): The campaign related to the activity. [optional]  # noqa: E501
            case_id (str, none_type): The case related to the activity. [optional]  # noqa: E501
            asset_id (str, none_type): The asset related to the activity. [optional]  # noqa: E501
            contract_id (str, none_type): The contract related to the activity. [optional]  # noqa: E501
            product_id (str, none_type): The product related to the activity. [optional]  # noqa: E501
            solution_id (str, none_type): The solution related to the activity. [optional]  # noqa: E501
            custom_object_id (str, none_type): The custom object related to the activity. [optional]  # noqa: E501
            title (str, none_type): The title of the activity. [optional]  # noqa: E501
            description (str, none_type): A description of the activity. [optional]  # noqa: E501
            note (str, none_type): An internal note about the activity. [optional]  # noqa: E501
            location (str, none_type): The location of the activity. [optional]  # noqa: E501
            location_address (Address): [optional]  # noqa: E501
            all_day_event (bool, none_type): Whether the Activity is an all day event or not. [optional]  # noqa: E501
            private (bool, none_type): Whether the Activity is private or not. [optional]  # noqa: E501
            group_event (bool, none_type): Whether the Activity is a group event or not. [optional]  # noqa: E501
            event_sub_type (str, none_type): The sub type of the group event. [optional]  # noqa: E501
            group_event_type (str, none_type): The type of the group event. [optional]  # noqa: E501
            child (bool, none_type): Whether the activity is a child of another activity or not. [optional]  # noqa: E501
            archived (bool, none_type): Whether the activity is archived or not. [optional]  # noqa: E501
            deleted (bool, none_type): Whether the activity is deleted or not. [optional]  # noqa: E501
            show_as (str, none_type): [optional]  # noqa: E501
            done (bool, none_type): Whether the Activity is done or not. [optional]  # noqa: E501
            start_datetime (str, none_type): The start date and time of the activity. [optional]  # noqa: E501
            end_datetime (str, none_type): The end date and time of the activity. [optional]  # noqa: E501
            duration_minutes (int, none_type): The duration of the activity in minutes. [optional]  # noqa: E501
            activity_date (str, none_type): The date of the activity. [optional]  # noqa: E501
            end_date (str, none_type): The end date of the activity. [optional]  # noqa: E501
            recurrent (bool): Whether the activity is recurrent or not. [optional]  # noqa: E501
            reminder_datetime (str, none_type): The date and time of the reminder. [optional]  # noqa: E501
            reminder_set (bool, none_type): Whether the reminder is set or not. [optional]  # noqa: E501
            video_conference_url (str, none_type): The URL of the video conference. [optional]  # noqa: E501
            video_conference_id (str, none_type): The ID of the video conference. [optional]  # noqa: E501
            custom_fields ([CustomField]): Custom fields of the activity. [optional]  # noqa: E501
            attendees ([ActivityAttendee]): [optional]  # noqa: E501
            custom_mappings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): When custom mappings are configured on the resource, the result is included here.. [optional]  # noqa: E501
            updated_by (str, none_type): The user who last updated the activity. [optional]  # noqa: E501
            created_by (str, none_type): The user who created the activity. [optional]  # noqa: E501
            updated_at (str, none_type): The date and time when the activity was last updated. [optional]  # noqa: E501
            created_at (str, none_type): The date and time when the activity was created. [optional]  # noqa: E501
            pass_through (PassThroughBody): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.type = type
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, type, *args, **kwargs):  # noqa: E501
        """Activity - a model defined in OpenAPI

        Args:
            type (str, none_type): The type of the activity

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): The unique identifier of the activity. [optional]  # noqa: E501
            downstream_id (str, none_type): The third-party API ID of original entity. [optional]  # noqa: E501
            activity_datetime (str, none_type): The date and time of the activity. [optional]  # noqa: E501
            duration_seconds (int, none_type): The duration of the activity in seconds. [optional]  # noqa: E501
            user_id (str, none_type): The user related to the activity. [optional]  # noqa: E501
            account_id (str, none_type): The account related to the activity. [optional]  # noqa: E501
            contact_id (str, none_type): The contact related to the activity. [optional]  # noqa: E501
            company_id (str, none_type): The company related to the activity. [optional]  # noqa: E501
            opportunity_id (str, none_type): The opportunity related to the activity. [optional]  # noqa: E501
            lead_id (str, none_type): The lead related to the activity. [optional]  # noqa: E501
            owner_id (str, none_type): The owner of the activity. [optional]  # noqa: E501
            campaign_id (str, none_type): The campaign related to the activity. [optional]  # noqa: E501
            case_id (str, none_type): The case related to the activity. [optional]  # noqa: E501
            asset_id (str, none_type): The asset related to the activity. [optional]  # noqa: E501
            contract_id (str, none_type): The contract related to the activity. [optional]  # noqa: E501
            product_id (str, none_type): The product related to the activity. [optional]  # noqa: E501
            solution_id (str, none_type): The solution related to the activity. [optional]  # noqa: E501
            custom_object_id (str, none_type): The custom object related to the activity. [optional]  # noqa: E501
            title (str, none_type): The title of the activity. [optional]  # noqa: E501
            description (str, none_type): A description of the activity. [optional]  # noqa: E501
            note (str, none_type): An internal note about the activity. [optional]  # noqa: E501
            location (str, none_type): The location of the activity. [optional]  # noqa: E501
            location_address (Address): [optional]  # noqa: E501
            all_day_event (bool, none_type): Whether the Activity is an all day event or not. [optional]  # noqa: E501
            private (bool, none_type): Whether the Activity is private or not. [optional]  # noqa: E501
            group_event (bool, none_type): Whether the Activity is a group event or not. [optional]  # noqa: E501
            event_sub_type (str, none_type): The sub type of the group event. [optional]  # noqa: E501
            group_event_type (str, none_type): The type of the group event. [optional]  # noqa: E501
            child (bool, none_type): Whether the activity is a child of another activity or not. [optional]  # noqa: E501
            archived (bool, none_type): Whether the activity is archived or not. [optional]  # noqa: E501
            deleted (bool, none_type): Whether the activity is deleted or not. [optional]  # noqa: E501
            show_as (str, none_type): [optional]  # noqa: E501
            done (bool, none_type): Whether the Activity is done or not. [optional]  # noqa: E501
            start_datetime (str, none_type): The start date and time of the activity. [optional]  # noqa: E501
            end_datetime (str, none_type): The end date and time of the activity. [optional]  # noqa: E501
            duration_minutes (int, none_type): The duration of the activity in minutes. [optional]  # noqa: E501
            activity_date (str, none_type): The date of the activity. [optional]  # noqa: E501
            end_date (str, none_type): The end date of the activity. [optional]  # noqa: E501
            recurrent (bool): Whether the activity is recurrent or not. [optional]  # noqa: E501
            reminder_datetime (str, none_type): The date and time of the reminder. [optional]  # noqa: E501
            reminder_set (bool, none_type): Whether the reminder is set or not. [optional]  # noqa: E501
            video_conference_url (str, none_type): The URL of the video conference. [optional]  # noqa: E501
            video_conference_id (str, none_type): The ID of the video conference. [optional]  # noqa: E501
            custom_fields ([CustomField]): Custom fields of the activity. [optional]  # noqa: E501
            attendees ([ActivityAttendee]): [optional]  # noqa: E501
            custom_mappings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): When custom mappings are configured on the resource, the result is included here.. [optional]  # noqa: E501
            updated_by (str, none_type): The user who last updated the activity. [optional]  # noqa: E501
            created_by (str, none_type): The user who created the activity. [optional]  # noqa: E501
            updated_at (str, none_type): The date and time when the activity was last updated. [optional]  # noqa: E501
            created_at (str, none_type): The date and time when the activity was created. [optional]  # noqa: E501
            pass_through (PassThroughBody): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.type = type
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from apideck.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from apideck.exceptions import ApiAttributeError


def lazy_import():
    from apideck.model.auth_type import AuthType
    from apideck.model.connection_configuration import ConnectionConfiguration
    from apideck.model.connection_state import ConnectionState
    from apideck.model.custom_mapping import CustomMapping
    from apideck.model.form_field import FormField
    from apideck.model.integration_state import IntegrationState
    from apideck.model.o_auth_grant_type import OAuthGrantType
    from apideck.model.webhook_subscription import WebhookSubscription
    globals()['AuthType'] = AuthType
    globals()['ConnectionConfiguration'] = ConnectionConfiguration
    globals()['ConnectionState'] = ConnectionState
    globals()['CustomMapping'] = CustomMapping
    globals()['FormField'] = FormField
    globals()['IntegrationState'] = IntegrationState
    globals()['OAuthGrantType'] = OAuthGrantType
    globals()['WebhookSubscription'] = WebhookSubscription


class Connection(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('status',): {
            'LIVE': "live",
            'UPCOMING': "upcoming",
            'REQUESTED': "requested",
        },
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'id': (str,),  # noqa: E501
            'service_id': (str,),  # noqa: E501
            'name': (str,),  # noqa: E501
            'tag_line': (str,),  # noqa: E501
            'unified_api': (str,),  # noqa: E501
            'state': (ConnectionState,),  # noqa: E501
            'integration_state': (IntegrationState,),  # noqa: E501
            'auth_type': (AuthType,),  # noqa: E501
            'oauth_grant_type': (OAuthGrantType,),  # noqa: E501
            'status': (str,),  # noqa: E501
            'enabled': (bool,),  # noqa: E501
            'website': (str,),  # noqa: E501
            'icon': (str,),  # noqa: E501
            'logo': (str,),  # noqa: E501
            'authorize_url': (str, none_type,),  # noqa: E501
            'revoke_url': (str, none_type,),  # noqa: E501
            'settings': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type,),  # noqa: E501
            'metadata': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type,),  # noqa: E501
            'form_fields': ([FormField],),  # noqa: E501
            'configuration': ([ConnectionConfiguration],),  # noqa: E501
            'configurable_resources': ([str],),  # noqa: E501
            'resource_schema_support': ([str],),  # noqa: E501
            'resource_settings_support': ([str],),  # noqa: E501
            'validation_support': (bool,),  # noqa: E501
            'schema_support': (bool,),  # noqa: E501
            'settings_required_for_authorization': ([str],),  # noqa: E501
            'subscriptions': ([WebhookSubscription],),  # noqa: E501
            'has_guide': (bool,),  # noqa: E501
            'created_at': (float,),  # noqa: E501
            'custom_mappings': ([CustomMapping],),  # noqa: E501
            'updated_at': (float, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'id': 'id',  # noqa: E501
        'service_id': 'service_id',  # noqa: E501
        'name': 'name',  # noqa: E501
        'tag_line': 'tag_line',  # noqa: E501
        'unified_api': 'unified_api',  # noqa: E501
        'state': 'state',  # noqa: E501
        'integration_state': 'integration_state',  # noqa: E501
        'auth_type': 'auth_type',  # noqa: E501
        'oauth_grant_type': 'oauth_grant_type',  # noqa: E501
        'status': 'status',  # noqa: E501
        'enabled': 'enabled',  # noqa: E501
        'website': 'website',  # noqa: E501
        'icon': 'icon',  # noqa: E501
        'logo': 'logo',  # noqa: E501
        'authorize_url': 'authorize_url',  # noqa: E501
        'revoke_url': 'revoke_url',  # noqa: E501
        'settings': 'settings',  # noqa: E501
        'metadata': 'metadata',  # noqa: E501
        'form_fields': 'form_fields',  # noqa: E501
        'configuration': 'configuration',  # noqa: E501
        'configurable_resources': 'configurable_resources',  # noqa: E501
        'resource_schema_support': 'resource_schema_support',  # noqa: E501
        'resource_settings_support': 'resource_settings_support',  # noqa: E501
        'validation_support': 'validation_support',  # noqa: E501
        'schema_support': 'schema_support',  # noqa: E501
        'settings_required_for_authorization': 'settings_required_for_authorization',  # noqa: E501
        'subscriptions': 'subscriptions',  # noqa: E501
        'has_guide': 'has_guide',  # noqa: E501
        'created_at': 'created_at',  # noqa: E501
        'custom_mappings': 'custom_mappings',  # noqa: E501
        'updated_at': 'updated_at',  # noqa: E501
    }

    read_only_vars = {
        'id',  # noqa: E501
        'service_id',  # noqa: E501
        'name',  # noqa: E501
        'tag_line',  # noqa: E501
        'unified_api',  # noqa: E501
        'status',  # noqa: E501
        'website',  # noqa: E501
        'icon',  # noqa: E501
        'logo',  # noqa: E501
        'authorize_url',  # noqa: E501
        'revoke_url',  # noqa: E501
        'form_fields',  # noqa: E501
        'configurable_resources',  # noqa: E501
        'resource_schema_support',  # noqa: E501
        'resource_settings_support',  # noqa: E501
        'validation_support',  # noqa: E501
        'schema_support',  # noqa: E501
        'settings_required_for_authorization',  # noqa: E501
        'subscriptions',  # noqa: E501
        'has_guide',  # noqa: E501
        'created_at',  # noqa: E501
        'updated_at',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """Connection - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): The unique identifier of the connection.. [optional]  # noqa: E501
            service_id (str): The ID of the service this connection belongs to.. [optional]  # noqa: E501
            name (str): The name of the connection. [optional]  # noqa: E501
            tag_line (str): [optional]  # noqa: E501
            unified_api (str): The unified API category where the connection belongs to.. [optional]  # noqa: E501
            state (ConnectionState): [optional]  # noqa: E501
            integration_state (IntegrationState): [optional]  # noqa: E501
            auth_type (AuthType): [optional]  # noqa: E501
            oauth_grant_type (OAuthGrantType): [optional]  # noqa: E501
            status (str): Status of the connection.. [optional]  # noqa: E501
            enabled (bool): Whether the connection is enabled or not. You can enable or disable a connection using the Update Connection API.. [optional]  # noqa: E501
            website (str): The website URL of the connection. [optional]  # noqa: E501
            icon (str): A visual icon of the connection, that will be shown in the Vault. [optional]  # noqa: E501
            logo (str): The logo of the connection, that will be shown in the Vault. [optional]  # noqa: E501
            authorize_url (str, none_type): The OAuth redirect URI. Redirect your users to this URI to let them authorize your app in the connector's UI. Before you can use this URI, you must add `redirect_uri` as a query parameter to the `authorize_url`. Be sure to URL encode the `redirect_uri` part. Your users will be redirected to this `redirect_uri` after they granted access to your app in the connector's UI.. [optional]  # noqa: E501
            revoke_url (str, none_type): The OAuth revoke URI. Redirect your users to this URI to revoke this connection. Before you can use this URI, you must add `redirect_uri` as a query parameter. Your users will be redirected to this `redirect_uri` after they granted access to your app in the connector's UI.. [optional]  # noqa: E501
            settings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): Connection settings. Values will persist to `form_fields` with corresponding id. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): Attach your own consumer specific metadata. [optional]  # noqa: E501
            form_fields ([FormField]): The settings that are wanted to create a connection.. [optional]  # noqa: E501
            configuration ([ConnectionConfiguration]): [optional]  # noqa: E501
            configurable_resources ([str]): [optional]  # noqa: E501
            resource_schema_support ([str]): [optional]  # noqa: E501
            resource_settings_support ([str]): [optional]  # noqa: E501
            validation_support (bool): [optional]  # noqa: E501
            schema_support (bool): [optional]  # noqa: E501
            settings_required_for_authorization ([str]): List of settings that are required to be configured on integration before authorization can occur. [optional]  # noqa: E501
            subscriptions ([WebhookSubscription]): [optional]  # noqa: E501
            has_guide (bool): Whether the connector has a guide available in the developer docs or not (https://docs.apideck.com/connectors/{service_id}/docs/consumer+connection).. [optional]  # noqa: E501
            created_at (float): [optional]  # noqa: E501
            custom_mappings ([CustomMapping]): List of custom mappings configured for this connection. [optional]  # noqa: E501
            updated_at (float, none_type): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """Connection - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): The unique identifier of the connection.. [optional]  # noqa: E501
            service_id (str): The ID of the service this connection belongs to.. [optional]  # noqa: E501
            name (str): The name of the connection. [optional]  # noqa: E501
            tag_line (str): [optional]  # noqa: E501
            unified_api (str): The unified API category where the connection belongs to.. [optional]  # noqa: E501
            state (ConnectionState): [optional]  # noqa: E501
            integration_state (IntegrationState): [optional]  # noqa: E501
            auth_type (AuthType): [optional]  # noqa: E501
            oauth_grant_type (OAuthGrantType): [optional]  # noqa: E501
            status (str): Status of the connection.. [optional]  # noqa: E501
            enabled (bool): Whether the connection is enabled or not. You can enable or disable a connection using the Update Connection API.. [optional]  # noqa: E501
            website (str): The website URL of the connection. [optional]  # noqa: E501
            icon (str): A visual icon of the connection, that will be shown in the Vault. [optional]  # noqa: E501
            logo (str): The logo of the connection, that will be shown in the Vault. [optional]  # noqa: E501
            authorize_url (str, none_type): The OAuth redirect URI. Redirect your users to this URI to let them authorize your app in the connector's UI. Before you can use this URI, you must add `redirect_uri` as a query parameter to the `authorize_url`. Be sure to URL encode the `redirect_uri` part. Your users will be redirected to this `redirect_uri` after they granted access to your app in the connector's UI.. [optional]  # noqa: E501
            revoke_url (str, none_type): The OAuth revoke URI. Redirect your users to this URI to revoke this connection. Before you can use this URI, you must add `redirect_uri` as a query parameter. Your users will be redirected to this `redirect_uri` after they granted access to your app in the connector's UI.. [optional]  # noqa: E501
            settings ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): Connection settings. Values will persist to `form_fields` with corresponding id. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}, none_type): Attach your own consumer specific metadata. [optional]  # noqa: E501
            form_fields ([FormField]): The settings that are wanted to create a connection.. [optional]  # noqa: E501
            configuration ([ConnectionConfiguration]): [optional]  # noqa: E501
            configurable_resources ([str]): [optional]  # noqa: E501
            resource_schema_support ([str]): [optional]  # noqa: E501
            resource_settings_support ([str]): [optional]  # noqa: E501
            validation_support (bool): [optional]  # noqa: E501
            schema_support (bool): [optional]  # noqa: E501
            settings_required_for_authorization ([str]): List of settings that are required to be configured on integration before authorization can occur. [optional]  # noqa: E501
            subscriptions ([WebhookSubscription]): [optional]  # noqa: E501
            has_guide (bool): Whether the connector has a guide available in the developer docs or not (https://docs.apideck.com/connectors/{service_id}/docs/consumer+connection).. [optional]  # noqa: E501
            created_at (float): [optional]  # noqa: E501
            custom_mappings ([CustomMapping]): List of custom mappings configured for this connection. [optional]  # noqa: E501
            updated_at (float, none_type): [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import sys
import unittest

import apideck
from apideck.model.address import Address
from apideck.model.bank_account import BankAccount
from apideck.model.currency import Currency
from apideck.model.custom_field import CustomField
from apideck.model.deprecated_linked_tracking_category import DeprecatedLinkedTrackingCategory
from apideck.model.invoice_line_item import InvoiceLineItem
from apideck.model.linked_customer import LinkedCustomer
from apideck.model.linked_ledger_account import LinkedLedgerAccount
from apideck.model.linked_tracking_categories import LinkedTrackingCategories
from apideck.model.pass_through_body import PassThroughBody
globals()['Address'] = Address
globals()['BankAccount'] = BankAccount
globals()['Currency'] = Currency
globals()['CustomField'] = CustomField
globals()['DeprecatedLinkedTrackingCategory'] = DeprecatedLinkedTrackingCategory
globals()['InvoiceLineItem'] = InvoiceLineItem
globals()['LinkedCustomer'] = LinkedCustomer
globals()['LinkedLedgerAccount'] = LinkedLedgerAccount
globals()['LinkedTrackingCategories'] = LinkedTrackingCategories
globals()['PassThroughBody'] = PassThroughBody
from apideck.model.invoice import Invoice


class TestInvoice(unittest.TestCase):
    """Invoice unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testInvoice(self):
        """Test Invoice"""
        # FIXME: construct object with mandatory attributes with example values
        # model = Invoice()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
